// --------------------------------------------------------------------
// Author  : mashimonator
// Create  : 2010/03/17
// Update  : 2010/03/17
//         : 2010/03/19 ページ印刷に対応
//         : 2012/09/06 色々と怠慢があったため修正
// Description : 画像を遅延ロードする
// --------------------------------------------------------------------

var lazyload = {
	//-----------------------------------------
	// settings
	//-----------------------------------------
	config : {
		cls : '', // クラス指定する場合のクラス名（未指定の場合は全画像が対象）
		blank : 'img/blank.gif', // blank画像のパス
		fade : true, // 画像をフェードインする
		speed : 50, // フェードインの実行間隔（ミリ秒単位）
		step : 0.5 // フェードインの加算値
	},
	storage : {
		screenHeight : null,
		imgs : []
	},
	initialize : function() {
		// 画面サイズを保持
		var size = lazyload.getScreenSize();
		lazyload.storage.screenHeight = size.height;
		// img要素を取得
		if ( lazyload.config.cls ) {
			var imgs = lazyload.getElementsByClassName(lazyload.config.cls);
		} else {
			var imgs = document.getElementsByTagName('IMG');
		}
		for (var i = 0, len = imgs.length; i < len; i++) {
			// 遅延ロード
			lazyload.setLazy(imgs[i]);
		}
		// 画像プリロード
		lazyload.addEvent(window, 'load', lazyload.preLoadImages);
		// スクロールイベントをセット
		lazyload.addEvent(window, 'scroll', lazyload.scroll);
	},
	setLazy : function( img ) {
		// 元の画像パスを保持
		img.orgSrc = img.src;
		// blank画像に置き換え
		img.src = lazyload.config.blank;
		// 要素を保持
		lazyload.storage.imgs[lazyload.storage.imgs.length] = {'img' : img, 'top' : null, 'bottom' : null, 'loaded' : false, 'executed' : false};
		// 透明度をセット
		lazyload.setOpacity(img, 0);
	},
	scroll : function() {
		// 現在の画面表示位置を取得
		var currentTop = window.pageYOffset || document.documentElement.scrollTop || document.body.scrollTop || 0;
		// 非表示（遅延ロード対象）画像を取得
		var imgs = lazyload.storage.imgs;
		for (var i = 0, len = imgs.length; i < len; i++) {
			// 画面内の要素を対象に処理
			if ( ((currentTop < imgs[i].bottom && imgs[i].bottom < (currentTop+lazyload.storage.screenHeight))
			|| (currentTop < imgs[i].top && imgs[i].top < (currentTop+lazyload.storage.screenHeight))) && !imgs[i].executed ) {
				if ( imgs[i].loaded ) {
					// 画像表示
					lazyload.show(imgs[i]);
				} else {
					// 代替画像表示
					lazyload.setOpacity(imgs[i].img, 10);
				}
			}
		}
	},
	show : function(item) {
		// fade用関数
		var fadeIn = function(item){
			return function(){
				// 不透明度取得
				var opacity = lazyload.getOpacity(item.img);
				if ( opacity < 10 ) {
					// 透過処理
					lazyload.setOpacity(item.img, (opacity + lazyload.config.step));
					// 再帰呼び出し
					window.setTimeout(fadeIn, lazyload.config.speed);
				} else {
					// 表示済みフラグをセット
					item.executed = true;
				}
			}
		}(item);
		// 表示
		switch ( lazyload.config.fade ) {
			case true:
				fadeIn();
				break;
			default:
				// 表示済みフラグをセット
				item.executed = true;
				lazyload.setOpacity(item.img, 10);
				break;
		}
	},
	preLoadImages : function(len) {
		if ( typeof len != 'number') {
			len = 0;
		}
		// 対象要素取得
		var imgs = lazyload.storage.imgs;
		if ( imgs.length > len ) {
			// preload
			var preLoad = new Image();
			preLoad.src = imgs[len].img.orgSrc;
			preLoad.onload = function(len, item){
				return function(){
					// 印刷用クラスセット
					if ( !lazyload.config.cls || lazyload.config.cls == '' ) {
						item.img.className += ' lazyLoadPrint';
					}
					// 元画像セット
					item.img.src = item.img.orgSrc;
					// ロード済みフラグをセット
					item.loaded = true;
					// 現在の画面表示位置を取得
					var currentTop = window.pageYOffset || document.documentElement.scrollTop || document.body.scrollTop || 0;
					// 画像の位置を取得
					var pos = lazyload.getElementPosition(item.img);
					item.top = pos.top; 
					// 画像のサイズ取得
					var bottom = pos.top+this.height;
					item.bottom = bottom;
					// 画面内の要素の場合
					if ( (currentTop < bottom && bottom < (currentTop+lazyload.storage.screenHeight))
					|| (currentTop < item.top && item.top < (currentTop+lazyload.storage.screenHeight)) ) {
						// 画像表示
						lazyload.show(item);
					} else {
						lazyload.setOpacity(item.img, 0);
					}
					lazyload.preLoadImages(len+1);
				}
			}(len, imgs[len]);
		} else {
			// 印刷用スタイルセット
			if ( !lazyload.config.cls || lazyload.config.cls == '' ) {
				lazyload.addPrintCssRule('.lazyLoadPrint', 'filter:alpha(opacity=100)!important; opacity:1.0!important;');
			} else {
				lazyload.addPrintCssRule('.' + lazyload.config.cls, 'filter:alpha(opacity=100)!important; opacity:1.0!important;');
			}
		}
	},
	addPrintCssRule : function ( selector, declarations ) {
		var style = document.createElement('style');
		style.setAttribute('type', 'text/css');
		style.setAttribute('media', 'print');
		document.getElementsByTagName('head')[0].appendChild(style);
		if ( lazyload.isIE() ) {
			style.styleSheet.addRule( selector, declarations );
		} else {
			style.sheet.insertRule(selector + '{' + declarations + '}', 0);
		}
	},
	isIE : function() {
		return IE='\v'=='v';
	},
	getOpacity : function( element ) {
		if ( element.alpha ) {
				return element.alpha / 10;
		} else if ( element.style ) {
			if ( element.style.opacity ) {
				return element.style.opacity * 10;
			}
		} 
		return 0;
	},
	setOpacity : function( element, opacity ) {
		element.style.opacity = opacity / 10;
		element.style.filter = 'alpha(opacity=' + opacity * 10 + ')';
		element.alpha = opacity * 10;
	},
	getScreenSize : function() {
		var w = 0;
		var h = 0;
		if ( window.innerWidth ) {
			w = window.innerWidth;
		} else if ( document.documentElement && document.documentElement.clientWidth != 0 ) {
			w = document.documentElement.clientWidth;
		} else if ( document.body ) {
			w = document.body.clientWidth;
		}
		if ( window.innerHeight ) {
			h = window.innerHeight;
		} else if ( document.documentElement && document.documentElement.clientHeight != 0 ) {
			h = document.documentElement.clientHeight;
		} else if ( document.body ) {
			h = document.body.clientHeight;
		}
		return ({ 'width': w, 'height': h });
	},
	getElementPosition : function( element ) {
		var offsetTrail = (typeof element == 'string') ? document.getElementById(element): element;
		var x = 0;
		var y = 0;
		while (offsetTrail) {
			x += offsetTrail.offsetLeft;
			y += offsetTrail.offsetTop;
			offsetTrail = offsetTrail.offsetParent;
		}
		if ( navigator.userAgent.indexOf('Mac') != -1 && typeof document.body.leftMargin != 'undefined' ) {
			x += document.body.leftMargin;
			y += document.body.topMargin;
		}
		return ({ 'left': x, 'top': y });
	},
	getElementsByClassName : function( cls ) {
		var elements = new Array();
		if ( document.evaluate ) {
			var xpathExpression = '/descendant::*[contains(@class,"'+ cls + '")]';
			var iterator = document.evaluate(xpathExpression, document, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
			for (var i = 0, len = iterator.snapshotLength; i < len; i++) {
				var elm = iterator.snapshotItem(i);
				elements[elements.length] = elm;
			}
		} else if ( document.querySelectorAll ) {
			elements = document.querySelectorAll('.'+cls);
		} else {
			var nodes = document.all ? document.all : document.getElementsByTagName('*');
			for (var i = 0, len = nodes.length; i < len; i++) {
				var elm = nodes.item(i);
				if ( elm.className.match(cls) ) {
					elements[elements.length] = elm;
				}
			}
		}
		return elements;
	},
	addEvent : function( target, event, func ) {
		try {
			target.addEventListener(event, func, false);
		} catch (e) {
			target.attachEvent('on' + event, (function(el){return function(){func.call(el);};})(target));
		}
	}
}
var clListener = {
	isAlreadyDetect : false,
	isAlreadyExec : false,
	funcList : [],
	add : function( func ) {
		switch ( clListener.isAlreadyExec ) {
			case true:
				func.call();
				break;
			default:
				clListener.funcList.push( function(){ return func.call(); } );
				break;
		}
	},
	detectEvent : function() {
		// check
		switch ( clListener.isAlreadyDetect ) {
			case true:
				return;
			default:
				clListener.isAlreadyDetect = true;
				break;
		}
		// detect
		if ( document.addEventListener ) {
			// for Mozilla, Opera and webkit
			document.addEventListener('DOMContentLoaded', function(){
				document.removeEventListener('DOMContentLoaded', arguments.callee, false);
				clListener.exec();
			}, false);
		} else if ( document.readyState === 'complete' ) {
			// for IE
			document.detachEvent('onreadystatechange', arguments.callee);
			clListener.exec();
		}
	},
	exec : function() {
		switch ( clListener.isAlreadyExec ) {
			case false:
				clListener.isAlreadyExec = true;
				if ( clListener.funcList ) {
					for (var i = 0, len = clListener.funcList.length; i < len; i++) {
						clListener.funcList[i].apply();
					}
					clListener.funcList = null;
				}
				break;
		}
	}
}
clListener.detectEvent();
clListener.add(lazyload.initialize);